// SerializeXML.cpp: implementation of the CSerializeXML class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SerializeXML.h"

#include <istream>
#include <fstream>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSerializeXML::CSerializeXML()
{
}

CSerializeXML::~CSerializeXML()
{
}

bool CSerializeXML::SaveConfigurationToDisk(_bstr_t file, MainStruct* pMainStruct)
{
   bool bResult = false;

   const char* pcFileOutput = static_cast<const char*> (file);
   ofstream fOutput;

   // Open the file to be read.
   fOutput.open(pcFileOutput, ios::app | ios::out);

   fOutput << "<?xml version=\"1.0\"?>" << "\n";
   fOutput << "<MainStruct" << "\n\t";
   fOutput << "xmlns:schemaLocation=\"Test.xsd\">" << "\n";

   {  // first
      fOutput << "\t" << "<first>" << "\n";

      for (int i = 0; i < pMainStruct->first.size(); ++i)
      {
         fOutput << "\t\t" << "<firstStruct>" << "\n";
         fOutput << "\t\t\t" << "<someName>" << static_cast<const char*> (pMainStruct->first[i].someName) << "</someName>" << "\n";
         fOutput << "\t\t\t" << "<someNumber>" << pMainStruct->first[i].someNumber << "</someNumber>" << "\n";
         fOutput << "\t\t\t" << "<firstSub1>" << "\n";

         for (int j = 0; j < (pMainStruct->first[i]).firstSub1.size(); ++j)
         {
            fOutput << "\t\t\t\t" << "<FirstSub1Struct>" << "\n";
            fOutput << "\t\t\t\t\t" << "<someNumber>" << (pMainStruct->first[i]).firstSub1[j].someNumber << "</someNumber>" << "\n";
            fOutput << "\t\t\t\t\t" << "<someName>" << static_cast<const char*> ((pMainStruct->first[i]).firstSub1[j].someName) << "</someName>" << "\n";
            fOutput << "\t\t\t\t" << "</FirstSub1Struct>" << "\n";
         }

         fOutput << "\t\t\t" << "</firstSub1>" << "\n";
         fOutput << "\t\t\t" << "<firstSub2>" << "\n";

         for (int k = 0; k < (pMainStruct->first[i]).firstSub2.size(); ++k)
         {
            fOutput << "\t\t\t\t" << "<FirstSub2Struct>" << "\n";
            fOutput << "\t\t\t\t\t" << "<someNumber>" << (pMainStruct->first[i]).firstSub2[k].someNumber << "</someNumber>" << "\n";
            fOutput << "\t\t\t\t\t" << "<someName>" << static_cast<const char*> ((pMainStruct->first[i]).firstSub2[k].someName) << "</someName>" << "\n";
            fOutput << "\t\t\t\t" << "</FirstSub2Struct>" << "\n";
         }

         fOutput << "\t\t\t" << "</firstSub2>" << "\n";
         fOutput << "\t\t" << "</firstStruct>" << "\n";
      }

      fOutput << "\t" << "</first>" << "\n";
   }

   {  // second
      fOutput << "\t" << "<second>" << "\n";

      for (int i = 0; i < pMainStruct->second.size(); ++i)
      {
         fOutput << "\t\t" << "<SecondStruct>" << "\n";
         fOutput << "\t\t\t" << "<someNumber1>" << (pMainStruct->second[i]).someNumber1 << "</someNumber1>" << "\n";
         fOutput << "\t\t\t" << "<someNumber2>" << (pMainStruct->second[i]).someNumber2 << "</someNumber2>" << "\n";
         fOutput << "\t\t" << "</SecondStruct>" << "\n";
      }

      fOutput << "\t" << "</second>" << "\n";
   }

   fOutput << "</MainStruct>" << "\n";

   // Close all files.
   fOutput.close();

   return bResult;
}

bool CSerializeXML::LoadConfigurationFromDisk(_bstr_t file, MainStruct* pMainStruct)
{
   bool bResult = false;

	// Initialize COM
   ::CoInitialize(NULL);

	// Create an instance of the parser.
   HRESULT hr = m_plDomDocument.CreateInstance(CLSID_DOMDocument);

   if (FAILED(hr))
      throw "Unable to create XML parser object";

	// Load the XML document file...
   variant_t vResult;
   vResult = m_plDomDocument->load(CComVariant(static_cast<const char*> (file)));

   if (((bool) vResult) == TRUE)
   {
      // Now that the document is loaded, we need to initialize the root pointer.
      m_pDocRoot = m_plDomDocument->documentElement;
      BuildMainStruct(m_pDocRoot, *pMainStruct);
   }
   else
   {
      throw "Unable to load XML document into the parser";
   }

   return bResult;
}

void CSerializeXML::BuildMainStruct(IXMLDOMNodePtr pParent, MainStruct &mainStruct)
{
   for (IXMLDOMNodePtr pChild = pParent->firstChild;
      NULL != pChild; pChild = pChild->nextSibling)
   {
      if (strcmp(pChild->nodeName, "first") == 0)
      {
         for (IXMLDOMNodePtr pNode = pChild->firstChild;
            NULL != pNode; pNode = pNode->nextSibling)
         {
            FirstStruct firstStruct;

            BuildFirstStruct(pNode, firstStruct);
            mainStruct.first.push_back(firstStruct);
         }
      }
      else if (strcmp(pChild->nodeName, "second") == 0)
      {
         for (IXMLDOMNodePtr pNode = pChild->firstChild;
            NULL != pNode; pNode = pNode->nextSibling)
         {
            SecondStruct secondEntryStruct;

            BuildSecondStruct(pNode, secondEntryStruct);
            mainStruct.second.push_back(secondEntryStruct);
         }
      }
      else
      {
         continue; // Skip unknown types.
      }
   }
}

void CSerializeXML::BuildFirstStruct(IXMLDOMNodePtr pParent, FirstStruct &firstStruct)
{
   for (IXMLDOMNodePtr pChild = pParent->firstChild;
      NULL != pChild; pChild = pChild->nextSibling)
   {
      if (strcmp(pChild->nodeName, "someName") == 0)
      {
         firstStruct.someName = (_bstr_t) pChild->text;
      }
      else if (strcmp(pChild->nodeName, "someNumber") == 0)
      {
         firstStruct.someNumber = atoi(static_cast<const char*> ((_bstr_t) pChild->text));
      }
      else if (strcmp(pChild->nodeName, "firstSub1") == 0)
      {
         for (IXMLDOMNodePtr pNode = pChild->firstChild;
            NULL != pNode; pNode = pNode->nextSibling)
         {
            FirstSub1Struct firstSub1Struct;

            if (strcmp(pNode->nodeName, "FirstSub1Struct") == 0)
            {
               BuildFirstSub1Struct(pNode, firstSub1Struct);
               firstStruct.firstSub1.push_back(firstSub1Struct);
            }
         }
      }
      else if (strcmp(pChild->nodeName, "firstSub2") == 0)
      {
         for (IXMLDOMNodePtr pNode = pChild->firstChild;
            NULL != pNode; pNode = pNode->nextSibling)
         {
            FirstSub2Struct firstSub2Struct;

            if (strcmp(pNode->nodeName, "FirstSub2Struct") == 0)
            {
               BuildFirstSub2Struct(pNode, firstSub2Struct);
               firstStruct.firstSub2.push_back(firstSub2Struct);
            }
         }
      }
      else
      {
         continue; // Skip unknown types.
      }
   }
}

void CSerializeXML::BuildFirstSub1Struct(IXMLDOMNodePtr pParent, FirstSub1Struct &firstSub1Struct)
{
   for (IXMLDOMNodePtr pChild = pParent->firstChild;
      NULL != pChild; pChild = pChild->nextSibling)
   {
      if (strcmp(pChild->nodeName, "someName") == 0)
      {
         firstSub1Struct.someName = (_bstr_t) pChild->text;
      }
      else if (strcmp(pChild->nodeName, "someNumber") == 0)
      {
         firstSub1Struct.someNumber = atoi(static_cast<const char*> ((_bstr_t) pChild->text));
      }
      else
      {
         continue; // Skip unknown types.
      }
   }
}

void CSerializeXML::BuildFirstSub2Struct(IXMLDOMNodePtr pParent, FirstSub2Struct &firstSub2Struct)
{
   for (IXMLDOMNodePtr pChild = pParent->firstChild;
      NULL != pChild; pChild = pChild->nextSibling)
   {
      if (strcmp(pChild->nodeName, "someName") == 0)
      {
         firstSub2Struct.someName = (_bstr_t) pChild->text;
      }
      else if (strcmp(pChild->nodeName, "someNumber") == 0)
      {
         firstSub2Struct.someNumber = atoi(static_cast<const char*> ((_bstr_t) pChild->text));
      }
      else
      {
         continue; // Skip unknown types.
      }
   }
}

void CSerializeXML::BuildSecondStruct(IXMLDOMNodePtr pParent, SecondStruct &secondStruct)
{
   for (IXMLDOMNodePtr pChild = pParent->firstChild;
      NULL != pChild; pChild = pChild->nextSibling)
   {
      if (strcmp(pChild->nodeName, "someNumber1") == 0)
      {
         secondStruct.someNumber1 = atoi(static_cast<const char*> ((_bstr_t) pChild->text));
      }
      else if (strcmp(pChild->nodeName, "someNumber2") == 0)
      {
         secondStruct.someNumber2 = atoi(static_cast<const char*> ((_bstr_t) pChild->text));
      }
      else
      {
         continue; // Skip unknown types.
      }
   }
}
